%% Author 
% Oskar Lund, 29.11.17
% Thesis - Butterfly Robot

%% Description
% This script creates plots of the derived general function g(phi). 
% There are three shapes (circle, ellipse and butterfly) that are 
% simulated to confirm the authenticity of the expression. 

%% Parameters
sizeR   = 'big';      % choices: small-> R = 0.01m, big-> R = 10m
shape   = 'butterfly';  % choices: circle, ellipse or butterfly

%% Define angles and variables
N = 1000;
phi = linspace(0,2*pi,N);
fung = zeros(1,N);
vphi = zeros(1,N);

for i=1:N
    [fung(i),~,~] = getGs(phi(i),sizeR,shape);
    vphi(i) = mod(fung(i),2*pi);
end
varphi = unwrap(vphi);

%% Plotting
figure
    grid on
    hold on
    zoom on
    plot(phi,phi,'b','LineWidth',1.5);
    plot(phi,varphi,'r','LineWidth',1.5);
    ax = gca;
    ax.XTick = [0 pi/2 pi 3*pi/2 2*pi];
    ax.XTickLabels = ({'0','$$\frac{1}{2}\pi$$','$$\pi$$','$$\frac{3}{2}\pi$$','$$2\pi$$'});
    ax.YTick = [0 pi/2 pi 3*pi/2 2*pi];
    ax.YTickLabels = ({'0','$$\frac{1}{2}\pi$$','$$\pi$$','$$\frac{3}{2}\pi$$','$$2\pi$$'});
    ax.TickLabelInterpreter = ('latex');
    xlabel('$\phi$','Interpreter','latex');
    ylabel('$\varphi$','Interpreter','latex');
    title('Relationship between $\phi$ and $\varphi$','Interpreter','latex');
    h = legend('$\phi$','$\varphi(\phi)$', 'Location','SouthEast');
    set(h,'Interpreter','latex','FontSize',12);

%% External functions from BR script
function [delta,Ddelta,DDdelta,DDDdelta] = getDELTAs(phi,shape)
    Rf = 1;
    delta = Rf; Ddelta = 0; DDdelta = 0; DDDdelta = 0;
    if strcmpi(shape,'ellipse')
        a = 0.1095; 
        b = 0.0405;
        t2 = cos(phi);
        t3 = sin(phi);
        t4 = a.^2;
        t5 = b.^2;
        t6 = t2.^2;
        t7 = t4.*t6;
        t8 = t3.^2;
        t9 = t5.*t8;
        t10 = t7+t9;
        delta = a.*b.*1.0./sqrt(t10);
        Ddelta = a.*b.*1.0./t10.^(3.0./2.0).*(t2.*t3.*t4.*2.0-t2.*t3.*t5.*2.0).*(1.0./2.0);
        t11 = t4-t5;
        DDdelta = a.*b.*1.0./t10.^(5.0./2.0).*t11.*(t4.*t6.^2-t5.*t8.^2+t4.*t6.*t8.*2.0-t5.*t6.*t8.*2.0);
        t12 = phi.*2.0;
        t13 = cos(t12);
        t14 = t4.^2;
        t15 = sin(t12);
        t16 = t5.^2;
        t17 = phi.*4.0;
        t18 = sin(t17);
        t19 = phi.*6.0;
        t20 = sin(t19);
        DDDdelta = sqrt(2.0).*a.*b.*t11.*1.0./(t4+t5+t4.*t13-t5.*t13).^(7.0./2.0).*(t14.*t15.*4.3e1+t15.*t16.*4.3e1+t14.*t18.*2.0e1-t14.*t20-t16.*t18.*2.0e1-t16.*t20-t4.*t5.*t15.*1.5e2+t4.*t5.*t20.*2.0).*(1.0./4.0);
    end
    if strcmpi(shape,'butterfly')
        a = 0.1095; 
        b = 0.0405;
        t2 = phi.*2.0;
        t3 = cos(t2);
        delta = a-b.*t3;
        t4 = sin(t2);
        Ddelta = b.*t4.*2.0;
        DDdelta = b.*t3.*4.0;
        DDDdelta = b.*t4.*-8.0;
    end
end        
function [alpha,Dalpha,DDalpha] = getALPHAs(phi,shape)
    [delta,Ddelta,DDdelta,DDDdelta] = getDELTAs(phi,shape);
    t2 = delta;
    t3 = sin(phi);
    t4 = Ddelta;
    t5 = cos(phi);
    alpha = atan((t2.*t3-t4.*t5)./(t2.*t5+t3.*t4));
    if (phi > pi/2 && phi < 3*pi/2)
        alpha = alpha + pi;
    end
    if (phi > 3*pi/2)
        alpha = alpha + 2*pi;
    end
    t6 = t4.^2;
    t7 = t2.^2;
    t8 = t6+t7;
    t9 = DDdelta;
    Dalpha = (t6.*2.0+t7-t2.*t9)./t8;
    t10 = DDDdelta;
    DDalpha = -1.0./t8.^2.*(t2.*t4.*t9.^2.*-2.0+t2.*t4.*t6.*2.0+t2.*t6.*t10+t2.*t7.*t10+t4.*t6.*t9-t4.*t7.*t9.*3.0);
end
function [g,Dg,DDg] = getGs(phi,sizeR,shape)
    R = 10;    
    if strcmpi(sizeR,'small')
            R = 0.01;
    end
    [delta,Ddelta,DDdelta,~] = getDELTAs(phi,shape);
    [alpha,Dalpha,DDalpha] = getALPHAs(phi,shape);
    t2 = delta;
    t3 = alpha;
    g = atan2((R.*sin(t3)+t2.*sin(phi)),(R.*cos(t3)+t2.*cos(phi)));
    t4 = t2.^2;
    t5 = R.^2;
    t6 = phi-t3;
    t7 = cos(t6);
    t8 = Dalpha;
    t9 = R.*t2.*t7.*2.0;
    t10 = t4+t5+t9;
    t11 = 1.0./t10;
    t12 = Ddelta;
    t13 = sin(t6);
    t14 = DDalpha;
    t15 = t2.*t12.*2.0;
    t16 = R.*t7.*t12.*2.0;
    t17 = t5.*t8;
    t18 = R.*t12.*t13;
    t19 = R.*t2.*t7;
    t20 = R.*t2.*t7.*t8;
    t21 = t4+t17+t18+t19+t20;
    Dg = t11.*t21;
    DDg = t11.*(t15+t16+t5.*t14-R.*t2.*t13+R.*t13.*DDdelta+R.*t2.*t7.*t14+R.*t2.*t8.^2.*t13)-1.0./t10.^2.*t21.*(t15+t16+R.*t2.*t13.*(t8-1.0).*2.0);
end

     